"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = manyToManyRelationships;

var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));

function arraysAreEqual(array1, array2) {
  return array1.length === array2.length && array1.every(function (el, i) {
    return array2[i] === el;
  });
} // Given a `leftTable`, trace through the foreign key relations
// and identify a `junctionTable` and `rightTable`.
// Returns a list of data objects for these many-to-many relationships.


function manyToManyRelationships(leftTable, build) {
  var introspectionResultsByKind = build.pgIntrospectionResultsByKind,
      omit = build.pgOmit;
  return leftTable.foreignConstraints.filter(function (con) {
    return con.type === 'f';
  }).reduce(function (memoLeft, junctionLeftConstraint) {
    if (omit(junctionLeftConstraint, 'read') || omit(junctionLeftConstraint, 'manyToMany')) {
      return memoLeft;
    }

    var junctionTable = introspectionResultsByKind.classById[junctionLeftConstraint.classId];

    if (!junctionTable) {
      throw new Error("Could not find the table that referenced us (constraint: ".concat(junctionLeftConstraint.name, ")"));
    }

    if (omit(junctionTable, 'read') || omit(junctionTable, 'manyToMany')) {
      return memoLeft;
    }

    var memoRight = junctionTable.constraints.filter(function (con) {
      return con.id !== junctionLeftConstraint.id && // Don't follow the same constraint back to the left table
      con.type === 'f' && !omit(con, 'read') && !omit(con, 'manyToMany');
    }).reduce(function (memoRight, junctionRightConstraint) {
      var rightTable = junctionRightConstraint.foreignClass;

      if (omit(rightTable, 'read') || omit(rightTable, 'manyToMany')) {
        return memoRight;
      }

      var leftKeyAttributes = junctionLeftConstraint.foreignKeyAttributes;
      var junctionLeftKeyAttributes = junctionLeftConstraint.keyAttributes;
      var junctionRightKeyAttributes = junctionRightConstraint.keyAttributes;
      var rightKeyAttributes = junctionRightConstraint.foreignKeyAttributes; // Ensure keys were found

      if (!leftKeyAttributes.every(function (_) {
        return _;
      }) || !junctionLeftKeyAttributes.every(function (_) {
        return _;
      }) || !junctionRightKeyAttributes.every(function (_) {
        return _;
      }) || !rightKeyAttributes.every(function (_) {
        return _;
      })) {
        throw new Error('Could not find key columns!');
      } // Ensure keys can be read


      if (leftKeyAttributes.some(function (attr) {
        return omit(attr, 'read');
      }) || junctionLeftKeyAttributes.some(function (attr) {
        return omit(attr, 'read');
      }) || junctionRightKeyAttributes.some(function (attr) {
        return omit(attr, 'read');
      }) || rightKeyAttributes.some(function (attr) {
        return omit(attr, 'read');
      })) {
        return memoRight;
      } // Ensure both constraints are single-column
      // TODO: handle multi-column


      if (leftKeyAttributes.length > 1 || rightKeyAttributes.length > 1) {
        return memoRight;
      } // Ensure junction constraint keys are not unique (which would result in a one-to-one relation)


      var junctionLeftConstraintIsUnique = !!junctionTable.constraints.find(function (c) {
        return ['p', 'u'].includes(c.type) && arraysAreEqual(c.keyAttributeNums, junctionLeftKeyAttributes.map(function (attr) {
          return attr.num;
        }));
      });
      var junctionRightConstraintIsUnique = !!junctionTable.constraints.find(function (c) {
        return ['p', 'u'].includes(c.type) && arraysAreEqual(c.keyAttributeNums, junctionRightKeyAttributes.map(function (attr) {
          return attr.num;
        }));
      });

      if (junctionLeftConstraintIsUnique || junctionRightConstraintIsUnique) {
        return memoRight;
      }

      var allowsMultipleEdgesToNode = !junctionTable.constraints.find(function (c) {
        return ['p', 'u'].includes(c.type) && arraysAreEqual(c.keyAttributeNums.concat().sort(), [].concat((0, _toConsumableArray2["default"])(junctionLeftKeyAttributes.map(function (obj) {
          return obj.num;
        })), (0, _toConsumableArray2["default"])(junctionRightKeyAttributes.map(function (obj) {
          return obj.num;
        }))).sort());
      });
      return [].concat((0, _toConsumableArray2["default"])(memoRight), [{
        leftKeyAttributes: leftKeyAttributes,
        junctionLeftKeyAttributes: junctionLeftKeyAttributes,
        junctionRightKeyAttributes: junctionRightKeyAttributes,
        rightKeyAttributes: rightKeyAttributes,
        junctionTable: junctionTable,
        rightTable: rightTable,
        junctionLeftConstraint: junctionLeftConstraint,
        junctionRightConstraint: junctionRightConstraint,
        allowsMultipleEdgesToNode: allowsMultipleEdgesToNode
      }]);
    }, []);
    return [].concat((0, _toConsumableArray2["default"])(memoLeft), (0, _toConsumableArray2["default"])(memoRight));
  }, []);
}